# -*- coding: utf-8 -*-

import functools
import datetime
import time
import json
import twitter as twitter_client
import pyvirtualdisplay
from selenium.webdriver import Firefox
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException, NoSuchElementException, WebDriverException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

FB_COOKIES = [
    ('.facebook.com', True, '/', False, 1528484827, 'datr',
     'I2pYV-Oz2ThckOLbyp8cRXXG',),
    ('.facebook.com', True, '/', True, 1530650268, 'sb',
     'H9t2VxxIAXbVyLqKSUwRR_cU',),
    ('.facebook.com', True, '/', True, 1475354268, 'pl',
     'n',),
    ('.facebook.com', True, '/', True, 1530650268, 'lu',
     'ggy_OHZODZ3-7Nv7Gx1IpRfA',),
    ('.facebook.com', True, '/', False, 0, 'p',
     '-2',),
    ('.facebook.com', True, '/', True, 1475398107, 'c_user',
     '100012791070015',),
    ('.facebook.com', True, '/', False, 1475398107, 'fr',
     '0sLXKwFmUatXbV1wH.AWWnZx6tGps0Xng1CsRyjoano5E.BXdtsf.FJ.AAA.0.0.BXeiLb.AWWwxj9K',),
    ('.facebook.com', True, '/', True, 1475398107, 'xs',
     '109%3AOGDMcpiPSC-jTg%3A2%3A1467578268%3A-1',),
    ('.facebook.com', True, '/', False, 1475398107, 'csm',
     '2',),
    ('.facebook.com', True, '/', True, 1475398107, 's',
     'Aa5QUyqlefROekmK.BXeXec',),
    ('.facebook.com', True, '/', True, 0, 'presence',
     'EDvF3EtimeF1467622107EuserFA21B12791070015A2EstateFDutF1467622107514Et2F_5b_5dElm2FnullEuct2F1467621506BEtrFnullEtwF1738432816EatF1467622106881CEchFDp_5f1B12791070015F2CC',),
    ('.facebook.com', True, '/', False, 0, 'wd',
     '1146x737'),
]

def _firefox(binary_path=None, log_file=None):
    if binary_path is not None:
        log_handle = open(log_file, 'a') if log_file else None
        firefox_binary = FirefoxBinary(firefox_path=binary_path, log_file=log_handle)
    else:
        firefox_binary = None
    return Firefox(firefox_binary=firefox_binary)


def with_display(func):
    @functools.wraps(func)
    def wrapper(*args, **kwds):
        display = pyvirtualdisplay.Display(visible=0, size=(1024, 768))
        display.start()
        response = func(*args, **kwds)
        display.stop()
        return response
    return wrapper


def _gplus_status(browser):
    try:
        msg_elm = browser.find_element_by_css_selector("p b")
        msg_text = msg_elm.get_attribute('innerHTML').strip()
        return "inactive" if "404." in msg_text else "active"
    except NoSuchElementException:
        return "active"


def _gplus_posts(browser, pages=3):

    load_more_js = ("document"
                    ".querySelector('div[data-waitmessage=\"Wait while more posts are being loaded\"]')"
                    ".scrollIntoView(true);")

    for _ in range(pages):
        browser.execute_script(load_more_js)
        time.sleep(1.5)

    posts_query = "content div[aria-label*=posts] c-wiz[data-focus]"
    try:
        post_elms = WebDriverWait(browser, 10).until(
            EC.presence_of_all_elements_located((By.CSS_SELECTOR, posts_query))
        )
    except TimeoutException:
        return []

    posts = []
    focus_comments_js = "arguments[0].click();"
    for post_elm in post_elms:
        browser.execute_script(focus_comments_js, post_elm)
        try:
            _ = WebDriverWait(post_elm, 3).until(
                EC.presence_of_element_located((By.TAG_NAME, "c-wiz"))
            )
            posts.append(post_elm.get_attribute('outerHTML'))
        except TimeoutException:
            continue
    return posts


def gplus(account, pages=3, binary_path=None, log_file=None):
    account_url = "https://plus.google.com/{}".format(account)
    browser = _firefox(binary_path=binary_path, log_file=log_file)
    browser.get(account_url)
    status = _gplus_status(browser)

    body_html = browser.find_element_by_tag_name('body').get_attribute('outerHTML')
    posts = None

    print status

    if status == "active":
        posts = _gplus_posts(browser, pages=pages)

    browser.close()
    browser.quit()

    return {
        "account": account,
        "status": status,
        "body": body_html,
        "posts": posts,
        "timestamp": str(datetime.datetime.now())
    }


def _twitch_status(browser):
    try:
        msg_elm = browser.find_element_by_css_selector(".no-channel-message-container .message")
        msg_text = msg_elm.get_attribute('innerHTML').strip()
        return "inactive" if "The page could not be found" in msg_text else "active"
    except NoSuchElementException:
        return "active"


def twitch(account, secs=10, binary_path=None, log_file=None):
    account_url = "https://www.twitch.tv/{}/profile".format(account)
    browser = _firefox(binary_path=binary_path, log_file=log_file)
    browser.get(account_url)
    status = _twitch_status(browser)

    body_html = browser.find_element_by_tag_name('body').get_attribute('outerHTML')
    posts = None

    if status == "active":
        chat_btn_selector = "#right_col .rightcol-content ul.tabs li:first-child a"
        try:
            chat_btn = WebDriverWait(browser, 10).until(
                EC.presence_of_element_located((By.CSS_SELECTOR, chat_btn_selector))
            )
            chat_btn.click()
            time.sleep(secs)

            msgs_selector = "#right_col .rightcol-content .message-line.chat-line"
            msg_elms = browser.find_elements_by_css_selector(msgs_selector)
            posts = [p.get_attribute('outerHTML') for p in msg_elms if "admin" not in p.get_attribute('class')]
        except TimeoutException:
            pass

    browser.close()
    browser.quit()

    return {
        "account": account,
        "status": status,
        "body": body_html,
        "posts": posts,
        "timestamp": str(datetime.datetime.now())
    }


def _get_twitter_client(auth_params):
    return twitter_client.Twitter(auth=twitter_client.OAuth(
        auth_params.access_token,
        auth_params.access_secret,
        auth_params.consumer_key,
        auth_params.consumer_secret
    ))


def _twitter_status(account, client):
    try:
        response = client.users.lookup(screen_name=account)
        try:
            return "private" if response[0]["protected"] else "active"
        except (KeyError, IndexError):
            return None
    except twitter_client.api.TwitterHTTPError:
        return "inactive"


def twitter(account, twitter_auth_params, **kwargs):
    t_client = _get_twitter_client(twitter_auth_params)
    status = _twitter_status(account, t_client)

    body = None
    posts = None
    mentions = None

    if status == "active":
        body = t_client.statuses.user_timeline(screen_name=account, count=30, include_rts=1)
        if body:
            posts = [p["text"] for p in body]
        mentions = t_client.search.tweets(q="@" + account, result_type="recent", count=30)['statuses']

    return {
        "account": account,
        "status": status,
        "body": json.dumps(body),
        "posts": posts,
        "mentions": mentions,
        "timestamp": str(datetime.datetime.now())
    }


@with_display
def youtube(account, pages=3, binary_path=None, log_file=None):
    account_url = "https://www.youtube.com/user/{}".format(account)
    browser = _firefox(binary_path=binary_path, log_file=log_file)
    browser.get(account_url)
    status = _youtube_status(browser)

    body_html = browser.find_element_by_tag_name('body').get_attribute('outerHTML')
    posts = None

    browser.close()
    browser.quit()

    if status == "active":
        discussion_url = "https://www.youtube.com/user/{}/discussion".format(account)
        browser = _firefox(binary_path=binary_path, log_file=log_file)
        browser.get(discussion_url)

        try:
            for _ in range(pages):
                _ = WebDriverWait(browser, 10).until(
                    EC.presence_of_element_located((By.CSS_SELECTOR, "button[data-uix-load-more-post=true]"))
                )
                scroll_js = ("document"
                             ".querySelector('button[data-uix-load-more-post=true]')"
                             ".scrollIntoView(true);")
                browser.execute_script(scroll_js)
        except (NoSuchElementException, TimeoutException):
            pass

        try:
            comments = browser.find_elements_by_css_selector(".comment-thread-renderer")
            posts = [c.get_attribute('outerHTML') for c in comments]
        except NoSuchElementException:
            pass

        browser.close()
        browser.quit()

    return {
        "account": account,
        "status": status,
        "body": body_html,
        "posts": posts,
        "timestamp": str(datetime.datetime.now())
    }


def _youtube_status(browser):
    try:
        h2_elm = browser.find_element_by_css_selector(".channel-empty-message h2")
        h2_text = h2_elm.get_attribute('innerHTML').strip()
        return "inactive" if h2_text == "This channel does not exist." else "active"
    except NoSuchElementException:
        return "active"


def instagram(account, instagram_auth_params, **kwargs):
    account_url = "https://www.instagram.com/{}/".format(account)

    browser = _firefox(binary_path=binary_path, log_file=log_file)
    browser.get(account_url)
    status = _instagram_status(browser)

    body_elm = browser.find_element_by_tag_name('body')
    body_html = body_elm.get_attribute('outerHTML')
    posts = None

    if status == "active":

        post_links_selector = "article > div > div > div > a"
        post_elms = browser.find_elements_by_css_selector(post_links_selector)
        posts = []
        for a_post in post_elms:
            a_post.click()
            dialog = WebDriverWait(browser, 5).until(
                EC.presence_of_element_located((By.CSS_SELECTOR, "div[role=dialog]"))
            )
            dialog_body = dialog.find_element_by_tag_name("article")
            dialog_html = dialog_body.get_attribute('outerHTML')
            dialog_body.send_keys(Keys.ESCAPE)
            time.sleep(.5)
            posts.append(dialog_html)

    browser.close()
    browser.quit()

    return {
        "account": account,
        "status": status,
        "body": body_html,
        "posts": posts,
        "timestamp": str(datetime.datetime.now())
    }


def _instagram_status(browser):
    try:
        h2_elms = browser.find_elements_by_tag_name("h2")
        if len(h2_elms) == 0:
            return "active"
        if len(h2_elms) > 1:
            h2_text = h2_elms[1].get_attribute('innerHTML').strip()
            if h2_text == "This Account is Private":
                return "private"
        h2_text = h2_elms[0].get_attribute('innerHTML').strip()
        return "inactive" if h2_text == "Sorry, this page isn't available." else "active"
    except NoSuchElementException:
        return "active"


@with_display
def facebook(profileid, pages=3, binary_path=None, log_file=None):
    try:
        _ = int(profileid)
        account_url = "https://www.facebook.com/profile.php?id={}".format(profileid)
    except ValueError:
        account_url = "https://www.facebook.com/{}".format(profileid)

    browser = _firefox(binary_path=binary_path, log_file=log_file)
    browser.get(account_url)
    status = _facebook_status(browser)

    body_elm = browser.find_element_by_tag_name('body')
    body_html = body_elm.get_attribute('outerHTML')

    posts = None

    if status in ["active", "private"]:
        for domain, _, path, _, expire, name, value in FB_COOKIES:

            browser.add_cookie({
                "name": name,
                "value": value,
                "domain": domain,
                "path": path,
                "expires": expire
            })

        browser.get(account_url)

        try:
            scroll_js = ("document"
                         ".querySelector('#pagelet_timeline_recent > div:last-child')"
                         ".scrollIntoView(true);")
            for _ in range(pages):
                browser.execute_script(scroll_js)
                time.sleep(1)

            posts_selector = "#recent_capsule_container ol > div > div"
            post_elms = browser.find_elements_by_css_selector(posts_selector)
            posts = [p.get_attribute('outerHTML') for p in post_elms]
        except WebDriverException:
            posts = []

        body_elm = browser.find_element_by_tag_name('body')
        body_html = body_elm.get_attribute('outerHTML')

    browser.close()
    browser.quit()

    return {
        "account": profileid,
        "status": status,
        "body": body_html,
        "posts": posts,
        "timestamp": str(datetime.datetime.now())
    }


def _facebook_profileid_or_account(account, browser):
    account_url = "https://www.facebook.com/{}".format(account)
    browser.get(account_url)



def _facebook_status(browser):

    # The possibilities to test for here
    #   *  The account is public and visible, in which case there will be a
    #      span#fb-timeline-cover-name element
    #   *  The username was never active.  We check for this by looking for a
    #      h2 element with the text "Sorry, this page isn't available" in it
    #   *  The account exists, but we don't have permission to view it.  We check
    #      for this case by looking for a h2 tag with the text
    #      "Sorry, this content isn't available right now"
    #   *  Something world changing and unpredictable

    try:
        _ = browser.find_element_by_css_selector("span#fb-timeline-cover-name, #pagelet_timeline_main_column")
        return "active"
    except NoSuchElementException:
        pass

    try:
        h2_elm = browser.find_element_by_css_selector("h2")
        h2_text = h2_elm.get_attribute("innerHTML").strip()
        if "Sorry, this page isn't available" in h2_text:
            return "inactive"
        elif "Sorry, this content isn't available right now" in h2_text:
            return "private"
        else:
            return None
    except NoSuchElementException:
        return None
