import re
import dateutil
import doxscrapers.sites.base
from doxscrapers.types import PostTeaser, Post


class KdePasteDoxScraper(doxscrapers.sites.base.BaseDoxScraper):

    def __init__(self, marshaller_host):
        super(KdePasteDoxScraper, self).__init__(3, marshaller_host, "kdepaste")

    def pause(self):
        self.throttle(45)

    def items_in_index(self, proxy=None):
        """
        Fetches a list of PostTeaser objects, each representing a thread /
        posting on KdePaste.

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, or a list of zero or more PostTeaser objects.
        """
        self._throttler()

        document = self._dom_for_url("https://paste.kde.org/all")
        if document is None:
            return None
        paste_links = document.select("a.btn.btn-success")
        paste_urls = [a.get("href") for a in paste_links]
        items = [PostTeaser(u.split("/")[-1]) for u in paste_urls]
        return items


    def details_for_item(self, item, proxy=None):
        """
        Returns a Post object, describing the full contents of the
        post that was listed in an index of posts.

        Args:
            item -- a doxscrapers.types.PostTeaser object

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, and otherwise a Post
            object.
        """
        post_url = "https://paste.kde.org/{}".format(item.key)

        document = self._dom_for_url(post_url, proxy=proxy)
        if document is None:
            return None

        paste_tabs = document.select("a.btn.btn-success")
        raw_tab = paste_tabs[2]
        scrape_url = raw_tab.get("href")

        raw_text_req = self._request_url(scrape_url, proxy=proxy)
        if raw_text_req is None:
            return None

        post = Post(item.key, raw_text_req.text, post_url)
        post.scrape_url = scrape_url

        posted_details_elm = document.select(".pre-footer .span6")[0]
        posted_details_test = posted_details_elm.get_text()
        matches = re.search(r"Posted by (.*) at (.*)", posted_details_test)

        post.added_on = dateutil.parser.parse(matches.group(2))
        post.user = matches.group(1)

        return post
